import {
  ChordAnalysis,
  ChordBase,
  ChordComponent,
  ChordExtensionType,
  ChordModificationType,
} from './common';

export const getHtmlChordSymbol = (analysis: ChordAnalysis) => {
  let chordSymbol = '';

  if (analysis.base === ChordBase.MINOR) {
    chordSymbol += 'm';
  } else if (analysis.base === ChordBase.DIMINISHED) {
    chordSymbol += 'dim';
  } else if (analysis.base === ChordBase.AUGMENTED) {
    chordSymbol += 'aug';
  }

  const {
    extensions = [],
    modifications = [],
    omissions = [],
  } = analysis;

  if (extensions.length > 0) {
    extensions.forEach((extension) => {
      if (extension.component === ChordComponent.SIXTH) {
        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol += '<sup>6</sup>';
        }
      }
      if (extension.component === ChordComponent.SEVENTH) {
        if (extension.type === ChordExtensionType.DOMINANT) {
          chordSymbol += '<sup>7</sup>';
        }

        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol += '<sup>maj7</sup>';
        }

        if (extension.type === ChordExtensionType.DIMINISHED) {
          chordSymbol += '<sup>7</sup>';
        }
      }

      if (extension.component === ChordComponent.NINTH) {
        if (extension.type === ChordExtensionType.MINOR) {
          chordSymbol += '<sup>(♭9)</sup>';
        }

        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol += '<sup>(9)</sup>';
        }

        if (extension.type === ChordExtensionType.AUGMENTED) {
          chordSymbol += '<sup>(♯9)</sup>';
        }
      }

      if (extension.component === ChordComponent.ELEVENTH) {
        if (extension.type === ChordExtensionType.MINOR) {
          chordSymbol += '<sup>(♭11)</sup>';
        }

        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol += '<sup>(11)</sup>';
        }
      }

      if (extension.component === ChordComponent.THIRTEENTH) {
        if (extension.type === ChordExtensionType.MINOR) {
          chordSymbol += '<sup>(♭13)</sup>';
        }

        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol += '<sup>(13)</sup>';
        }

        if (extension.type === ChordExtensionType.AUGMENTED) {
          chordSymbol += '<sup>(♯13)</sup>';
        }
      }
    });
  }

  if (modifications.length > 0) {
    modifications.forEach((modification) => {
      if (modification.component === ChordComponent.FIFTH) {
        if (modification.type === ChordModificationType.LOWERED) {
          chordSymbol += '<sup>(♭5)</sup>';
        }
      }

      if (modification.component === ChordComponent.FOURTH) {
        if (modification.type === ChordModificationType.SUSPENDED) {
          chordSymbol += 'sus4';
        }
      }

      if (modification.component === ChordComponent.SECOND) {
        if (modification.type === ChordModificationType.SUSPENDED) {
          chordSymbol += 'sus2';
        }
      }
    });
  }

  if (omissions.length > 0) {
    omissions.forEach((omission) => {
      switch (omission) {
        case ChordComponent.THIRD:
          chordSymbol += '<sup>(no3)</sup>';
          break;
        case ChordComponent.FIFTH:
          chordSymbol += '<sup>(no5)</sup>';
          break;
        case ChordComponent.SEVENTH:
          chordSymbol += '<sup>(no7)</sup>';
          break;
        case ChordComponent.NINTH:
          chordSymbol += '<sup>(no9)</sup>';
          break;
        case ChordComponent.ELEVENTH:
          chordSymbol += '<sup>(no11)</sup>';
          break;
        default:
          break;
      }
    });
  }

  return chordSymbol;
};

export const getChordId = (analysis: ChordAnalysis) => (
  getHtmlChordSymbol(analysis)
    .replace(/<sup>(.+?)<\/sup>/g, '^$1')
    .replace(/♯/g, '#')
    .replace(/♭/g, 'b')
);

export const getChordName = (analysis: ChordAnalysis) => {
  const chordSymbol = [] as string[];

  if (analysis.base === ChordBase.MINOR) {
    chordSymbol.push('minor');
  } else if (analysis.base === ChordBase.DIMINISHED) {
    chordSymbol.push('diminished');
  } else if (analysis.base === ChordBase.AUGMENTED) {
    chordSymbol.push('augmented');
  }

  const {
    extensions = [],
    modifications = [],
    omissions = [],
  } = analysis;

  if (extensions.length > 0) {
    extensions.forEach((extension) => {
      if (extension.component === ChordComponent.SIXTH) {
        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol.push('sixth');
        }
      }
      if (extension.component === ChordComponent.SEVENTH) {
        if (extension.type === ChordExtensionType.DOMINANT) {
          chordSymbol.push('seventh');
        }

        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol.push('major seventh');
        }

        if (extension.type === ChordExtensionType.DIMINISHED) {
          chordSymbol.push('seventh');
        }
      }

      if (extension.component === ChordComponent.NINTH) {
        if (extension.type === ChordExtensionType.MINOR) {
          chordSymbol.push('minor ninth');
        }

        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol.push('ninth');
        }

        if (extension.type === ChordExtensionType.AUGMENTED) {
          chordSymbol.push('augmented ninth');
        }
      }

      if (extension.component === ChordComponent.ELEVENTH) {
        if (extension.type === ChordExtensionType.MINOR) {
          chordSymbol.push('minor eleventh');
        }

        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol.push('eleventh');
        }
      }

      if (extension.component === ChordComponent.THIRTEENTH) {
        if (extension.type === ChordExtensionType.MINOR) {
          chordSymbol.push('minor thirteenth');
        }

        if (extension.type === ChordExtensionType.MAJOR) {
          chordSymbol.push('thirteenth');
        }

        if (extension.type === ChordExtensionType.AUGMENTED) {
          chordSymbol.push('augmented thirteenth');
        }
      }
    });
  }

  if (modifications.length > 0) {
    modifications.forEach((modification) => {
      if (modification.component === ChordComponent.FIFTH) {
        if (modification.type === ChordModificationType.LOWERED) {
          chordSymbol.push('lowered fifth');
        }
      }

      if (modification.component === ChordComponent.FOURTH) {
        if (modification.type === ChordModificationType.SUSPENDED) {
          chordSymbol.push('suspended fourth');
        }
      }

      if (modification.component === ChordComponent.SECOND) {
        if (modification.type === ChordModificationType.SUSPENDED) {
          chordSymbol.push('suspended second');
        }
      }
    });
  }

  if (omissions.length > 0) {
    omissions.forEach((omission) => {
      switch (omission) {
        case ChordComponent.THIRD:
          chordSymbol.push('no third');
          break;
        case ChordComponent.FIFTH:
          chordSymbol.push('no fifth');
          break;
        case ChordComponent.SEVENTH:
          chordSymbol.push('no seventh');
          break;
        case ChordComponent.NINTH:
          chordSymbol.push('no ninth');
          break;
        case ChordComponent.ELEVENTH:
          chordSymbol.push('no eleventh');
          break;
        default:
          break;
      }
    });
  }

  return chordSymbol.join(' ');
};
